<?php

class CustomOcPostType {

	function __construct( $file ) {
		//Add hook to register OC custom post type
		add_action( 'init', array( &$this, 'copt_article_register_post_types' ) );

		//Add Hook to filter Custom post template on admin page
		add_action( 'admin_init', array( &$this, 'copt_admin_filter_page_template' ) );

	    add_filter( 'post_type_link', array( &$this, 'copt_custom_post_link' ), 10, 2 );

        add_filter( 'post_row_actions', array( &$this, 'copt_add_row_action'), 10 , 2 );
	}

	/*
	 * Function to setup args for and register OC article as custom post type
	 */
	function copt_article_register_post_types() {

        $oc = OpenContent::getInstance();

        $article_args = array(
			'public'        => true,
			'query_var'     => true,
			'menu_position' => 20,
			'menu_icon'     => EVERY_BASE . 'admin-style/images/article.png',
			'rewrite'       => array(
				'slug'       => $oc->getCustomPostTypeSlug(),
				'with_front' => false
			),
			'supports'      => array(
				'title', 'custom-fields'
			),
			'labels'        => array(
				'name'               => __( 'Articles', 'every' ),
				'singular_name'      => __( 'Article', 'every' ),
				'add_new'            => __( 'Add New Article', 'every' ),
				'add_new_item'       => __( 'Add New Article', 'every' ),
				'edit_item'          => __( 'Edit Article', 'every' ),
				'new_item'           => __( 'New Article', 'every' ),
				'view_item'          => __( 'View Article', 'every' ),
				'search_items'       => __( 'Search Articles', 'every' ),
				'not_found'          => __( 'No Articles Found', 'every' ),
				'not_found_in_trash' => __( 'No Articles Found In Trash', 'every' )
			),
			'taxonomies'    => array( 'category', 'post_tag' )
		);

		register_post_type( 'article', $article_args );

        add_action('generate_rewrite_rules', function ($wp_rewrite)
        {
            $rules = $wp_rewrite->rules;
            $rules["[^/]+/([^/]+)/?$"] = 'index.php?article=$matches[1]';
            $rules["([^/]+/)*([^/]+)/?$"] = 'index.php?article=$matches[2]';
            $wp_rewrite->rules = $rules;
        });
    }

    function copt_add_row_action( $actions ) {
        global $post;

        $uuid = get_post_meta( $post->ID, 'oc_uuid', true );
        $actions['purge'] = "<a data-uuid='". $uuid ."' class='purge_cache' href='#' class='purge_cache' title='" . __( 'Empty cache from this article', 'every' ) . "'>" . __( 'Purge Cache', 'every' ) . "</a>";
        return $actions;
    }

	/*
	 * Function will only register a filter for "get template"
	 */
	function copt_admin_filter_page_template() {
		// Filter page template
		add_filter( 'page_template', array( &$this, 'catch_oc_custom_post_template' ), 10, 1 );
	}

	/*
	 * A filter function that will help us detect when OC Article template is requested in admin
	 * When called, always return single-article
	 */
	function catch_oc_custom_post_template( $template ) {
		$template = '';
		$fields   = explode( '/', $template );

		if( count( $fields ) > 1 ){
			$template = $fields[count( $fields ) - 1];
		}

		if ( get_post_type() == 'article' ) {
			$template = get_theme_root() . '/single-article.php';
		}

		return $template;
	}

    function copt_custom_post_link( $post_link, $post ) {

        $cat_ttl  = 86400;

        if ( ! is_object( $post ) || $post->post_type != 'article' ) {
            return $post_link;
        }

        $terms_key = md5( $post->ID ) . '_category_term';
        if ( false === ( $terms = get_transient( $terms_key ) ) ) {
            $terms = wp_get_object_terms( $post->ID, 'category' );
            set_transient( $terms_key, $terms, $cat_ttl );
        }

        if ( !empty( $terms ) && !is_wp_error( $terms ) ) {

            $slug = [];
            $order = get_post_meta( $post->ID, 'url_term_order', true);

            foreach( $terms as $term ) {
                $category = $term->slug;

                if( false !== $order && is_array( $order ) ) {
                    $key = array_search( $term->term_id, $order );

                    if( false !== $key ) {
                        $slug[$key] = preg_replace("/\-([0-9]{1,})$/", "", $category);
                    } else {
                        array_push($slug, preg_replace("/\-([0-9]{1,})$/", "", $category) );
                    }
                } else {
                    array_push($slug, preg_replace("/\-([0-9]{1,})$/", "", $category) );
                }
            }

            ksort($slug);

            return str_replace( '_category_', join('/', $slug), $post_link );
        } else {
            return str_replace( '_category_', "article", $post_link );
        }
    }
}